@@ -438,6 +438,42 @@ apr_byte_t oidc_jwk_parse_json(apr_pool_t *pool, json_t *json, oidc_jwk_t **jwk,
 	return (*jwk != NULL);
 }
 
+apr_byte_t oidc_jwks_parse_json(apr_pool_t *pool, json_t *json, apr_array_header_t **jwk_list, oidc_jose_error_t *err) {
+	const json_t *keys = json_object_get(json, OIDC_JOSE_JWKS_KEYS_STR);
+	if ((keys == NULL) || (!json_is_array(keys))) {
+		oidc_jose_error(err, "JWKS did not contain \"" OIDC_JOSE_JWKS_KEYS_STR "\" array");
+		return FALSE;
+	}
+	*jwk_list = apr_array_make(pool, json_array_size(keys), sizeof(oidc_jwk_t *));
+	for (int i = 0; i < json_array_size(keys); i++) {
+		json_t *elem = json_array_get(keys, i);
+		if (elem == NULL)
+			continue;
+		oidc_jwk_t *jwk;
+		if (oidc_jwk_parse_json(pool, elem, &jwk, err) != TRUE) {
+			return FALSE;
+		}
+		APR_ARRAY_PUSH(*jwk_list, oidc_jwk_t *) = jwk;
+	}
+	return TRUE;
+}
+
+apr_byte_t oidc_is_jwk(json_t *json) {
+	const json_t *kty = json_object_get(json, OIDC_JOSE_JWK_KTY_STR);
+	if ((kty == NULL) || (!json_is_string(kty))) {
+		return FALSE;
+	}
+	return TRUE;
+}
+
+apr_byte_t oidc_is_jwks(json_t *json) {
+	const json_t *keys = json_object_get(json, OIDC_JOSE_JWKS_KEYS_STR);
+	if ((keys == NULL) || (!json_is_array(keys))) {
+		return FALSE;
+	}
+	return TRUE;
+}
+
 /*
  * convert a JWK struct to a JSON string
  */
@@ -820,27 +856,34 @@ static apr_byte_t oidc_jose_brotli_uncompress(apr_pool_t *pool, const char *inpu
 
 static apr_byte_t oidc_jose_zlib_compress(apr_pool_t *pool, const char *input, int input_len, char **output,
 					  int *output_len, oidc_jose_error_t *err) {
+	int status = Z_OK;
 	z_stream zlib;
+
 	zlib.zalloc = Z_NULL;
 	zlib.zfree = Z_NULL;
 	zlib.opaque = Z_NULL;
 	zlib.next_in = (Bytef *)input;
 	zlib.avail_in = input_len;
+
 	*output = apr_pcalloc(pool, input_len * 2);
 	zlib.next_out = (Bytef *)(*output);
 	zlib.avail_out = input_len * 2;
-	int deflate_status = Z_OK;
 
-	deflateInit(&zlib, Z_BEST_COMPRESSION);
-
-	deflate_status = deflate(&zlib, Z_FINISH);
-	if (deflate_status != Z_STREAM_END) {
-		oidc_jose_error(err, "deflate failed: %d", deflate_status);
+	status = deflateInit(&zlib, Z_BEST_COMPRESSION);
+	if (status != Z_OK) {
+		oidc_jose_error(err, "deflateInit() failed: %d", status);
 		return FALSE;
 	}
 
-	if (deflateEnd(&zlib) != Z_OK) {
-		oidc_jose_error(err, "deflateEnd failed");
+	status = deflate(&zlib, Z_FINISH);
+	if (status != Z_STREAM_END) {
+		oidc_jose_error(err, "deflate() failed: %d", status);
+		return FALSE;
+	}
+
+	status = deflateEnd(&zlib);
+	if (status != Z_OK) {
+		oidc_jose_error(err, "deflateEnd() failed: %d", status);
 		return FALSE;
 	}
 
@@ -853,45 +896,50 @@ static apr_byte_t oidc_jose_zlib_compress(apr_pool_t *pool, const char *input, i
 
 static apr_byte_t oidc_jose_zlib_uncompress(apr_pool_t *pool, const char *input, int input_len, char **output,
 					    int *output_len, oidc_jose_error_t *err) {
+	int status = Z_OK;
+	size_t len = OIDC_CJOSE_UNCOMPRESS_CHUNK;
+	char *tmp = NULL, *buf = apr_pcalloc(pool, len);
 	z_stream zlib;
+
 	zlib.zalloc = Z_NULL;
 	zlib.zfree = Z_NULL;
 	zlib.opaque = Z_NULL;
 	zlib.avail_in = (uInt)input_len;
 	zlib.next_in = (Bytef *)input;
 	zlib.total_out = 0;
-	size_t uncompLength = OIDC_CJOSE_UNCOMPRESS_CHUNK;
-	char *uncomp = (char *)apr_pcalloc(pool, uncompLength + 1);
 
-	inflateInit(&zlib);
-	int done = 1;
-	while (done) {
-		int inflate_status;
-		if (zlib.total_out >= OIDC_CJOSE_UNCOMPRESS_CHUNK) {
-			char *uncomp2 = (char *)apr_pcalloc(pool, uncompLength + OIDC_CJOSE_UNCOMPRESS_CHUNK);
-			_oidc_memcpy(uncomp2, uncomp, uncompLength);
-			uncompLength += OIDC_CJOSE_UNCOMPRESS_CHUNK;
-			uncomp = uncomp2;
-		}
-		zlib.next_out = (Bytef *)(uncomp + zlib.total_out);
-		zlib.avail_out = (uInt)uncompLength - zlib.total_out;
-		inflate_status = inflate(&zlib, Z_SYNC_FLUSH);
-		if (inflate_status == Z_STREAM_END) {
-			done = 0;
-		} else if (inflate_status != Z_OK) {
-			oidc_jose_error(err, "inflate failed: %d", inflate_status);
-			inflateEnd(&zlib);
-			return FALSE;
-		}
+	status = inflateInit(&zlib);
+	if (status != Z_OK) {
+		oidc_jose_error(err, "inflateInit() failed: %d", status);
+		return FALSE;
 	}
 
-	if (inflateEnd(&zlib) != Z_OK) {
-		oidc_jose_error(err, "inflateEnd failed");
+	while (status == Z_OK) {
+		if (zlib.total_out >= OIDC_CJOSE_UNCOMPRESS_CHUNK) {
+			tmp = apr_pcalloc(pool, len + OIDC_CJOSE_UNCOMPRESS_CHUNK);
+			_oidc_memcpy(tmp, buf, len);
+			len += OIDC_CJOSE_UNCOMPRESS_CHUNK;
+			buf = tmp;
+		}
+		zlib.next_out = (Bytef *)(buf + zlib.total_out);
+		zlib.avail_out = (uInt)len - zlib.total_out;
+		status = inflate(&zlib, Z_SYNC_FLUSH);
+	}
+
+	if (status != Z_STREAM_END) {
+		oidc_jose_error(err, "inflate() failed: %d", status);
+		inflateEnd(&zlib);
+		return FALSE;
+	}
+
+	status = inflateEnd(&zlib);
+	if (status != Z_OK) {
+		oidc_jose_error(err, "inflateEnd() failed: %d", status);
 		return FALSE;
 	}
 
 	*output_len = (int)zlib.total_out;
-	*output = uncomp;
+	*output = buf;
 
 	return TRUE;
 }
@@ -1112,7 +1160,7 @@ apr_byte_t oidc_jwt_encrypt(apr_pool_t *pool, oidc_jwt_t *jwe, oidc_jwk_t *jwk,
  */
 apr_byte_t oidc_jose_version_deprecated(apr_pool_t *pool) {
 	char *version = apr_pstrdup(pool, cjose_version());
-	return (strstr(version, OIDC_JOSE_CJOSE_VERSION_DEPRECATED) == version);
+	return (_oidc_strstr(version, OIDC_JOSE_CJOSE_VERSION_DEPRECATED) == version);
 }
 
 /*
@@ -1355,8 +1403,8 @@ static apr_byte_t _oidc_jwk_rsa_key_to_jwk(apr_pool_t *pool, EVP_PKEY *pkey, oid
 
 	*fp_len = key_spec.nlen + key_spec.elen;
 	*fp = apr_pcalloc(pool, *fp_len);
-	memcpy(*fp, key_spec.n, key_spec.nlen);
-	memcpy(*fp + key_spec.nlen, key_spec.e, key_spec.elen);
+	_oidc_memcpy(*fp, key_spec.n, key_spec.nlen);
+	_oidc_memcpy(*fp + key_spec.nlen, key_spec.e, key_spec.elen);
 
 	rv = TRUE;
 
@@ -1447,9 +1495,9 @@ static apr_byte_t _oidc_jwk_ec_key_to_jwk(apr_pool_t *pool, EVP_PKEY *pkey, oidc
 	apr_uint32_t b = htonl(crv);
 	*fp_len = sizeof(b) + ec_keyspec.xlen + ec_keyspec.ylen;
 	*fp = apr_pcalloc(pool, *fp_len);
-	memcpy(*fp, &b, sizeof(b));
-	memcpy(*fp + sizeof(b), ec_keyspec.x, ec_keyspec.xlen);
-	memcpy(*fp + sizeof(b) + ec_keyspec.xlen, ec_keyspec.y, ec_keyspec.ylen);
+	_oidc_memcpy(*fp, &b, sizeof(b));
+	_oidc_memcpy(*fp + sizeof(b), ec_keyspec.x, ec_keyspec.xlen);
+	_oidc_memcpy(*fp + sizeof(b) + ec_keyspec.xlen, ec_keyspec.y, ec_keyspec.ylen);
 
 	rv = TRUE;
 
