@@ -74,7 +74,7 @@ extern module AP_MODULE_DECLARE_DATA auth_openidc_module;
 #define OIDC_METADATA_CLIENT_SECRET "client_secret"
 #define OIDC_METADATA_CLIENT_SECRET_EXPIRES_AT "client_secret_expires_at"
 
-#define OIDC_METADATA_KEYS "keys"
+#define OIDC_METADATA_KEYS OIDC_JOSE_JWKS_KEYS_STR
 
 #define OIDC_METADATA_CLIENT_JWKS_URI "client_jwks_uri"
 #define OIDC_METADATA_ID_TOKEN_SIGNED_RESPONSE_ALG "id_token_signed_response_alg"
@@ -125,11 +125,11 @@ extern module AP_MODULE_DECLARE_DATA auth_openidc_module;
 static const char *oidc_metadata_issuer_to_filename(request_rec *r, const char *issuer) {
 
 	/* strip leading https:// */
-	char *p = strstr(issuer, "https://");
+	char *p = _oidc_strstr(issuer, "https://");
 	if (p == issuer) {
 		p = apr_pstrdup(r->pool, issuer + _oidc_strlen("https://"));
 	} else {
-		p = strstr(issuer, "http://");
+		p = _oidc_strstr(issuer, "http://");
 		if (p == issuer) {
 			p = apr_pstrdup(r->pool, issuer + _oidc_strlen("http://"));
 		} else {
@@ -142,7 +142,7 @@ static const char *oidc_metadata_issuer_to_filename(request_rec *r, const char *
 	if (p[n - 1] == OIDC_CHAR_FORWARD_SLASH)
 		p[n - 1] = '\0';
 
-	return oidc_util_escape_string(r, p);
+	return oidc_http_escape_string(r, p);
 }
 
 /*
@@ -152,7 +152,7 @@ static const char *oidc_metadata_filename_to_issuer(request_rec *r, const char *
 	char *result = apr_pstrdup(r->pool, filename);
 	char *p = strrchr(result, OIDC_CHAR_DOT);
 	*p = '\0';
-	p = oidc_util_unescape_string(r, result);
+	p = oidc_http_unescape_string(r, result);
 	return apr_psprintf(r->pool, "https://%s", p);
 }
 
@@ -544,9 +544,9 @@ static apr_byte_t oidc_metadata_client_register(request_rec *r, oidc_cfg *cfg, o
 	}
 
 	/* dynamically register the client with the specified parameters */
-	if (oidc_util_http_post_json(r, provider->registration_endpoint_url, data, NULL, provider->registration_token,
-				     provider->ssl_validate_server, response, &cfg->http_timeout_short,
-				     &cfg->outgoing_proxy, oidc_dir_cfg_pass_cookies(r), NULL, NULL, NULL) == FALSE) {
+	if (oidc_http_post_json(r, provider->registration_endpoint_url, data, NULL, provider->registration_token,
+				provider->ssl_validate_server, response, NULL, &cfg->http_timeout_short,
+				&cfg->outgoing_proxy, oidc_dir_cfg_pass_cookies(r), NULL, NULL, NULL) == FALSE) {
 		json_decref(data);
 		return FALSE;
 	}
@@ -571,16 +571,28 @@ static apr_byte_t oidc_metadata_jwks_retrieve_and_cache(request_rec *r, oidc_cfg
 	const char *url = (jwks_uri->signed_uri != NULL) ? jwks_uri->signed_uri : jwks_uri->uri;
 
 	/* get the JWKs from the specified URL with the specified parameters */
-	if (oidc_util_http_get(r, url, NULL, NULL, NULL, ssl_validate_server, &response, &cfg->http_timeout_long,
-			       &cfg->outgoing_proxy, oidc_dir_cfg_pass_cookies(r), NULL, NULL, NULL) == FALSE)
+	if (oidc_http_get(r, url, NULL, NULL, NULL, ssl_validate_server, &response, NULL, &cfg->http_timeout_long,
+			  &cfg->outgoing_proxy, oidc_dir_cfg_pass_cookies(r), NULL, NULL, NULL) == FALSE)
 		return FALSE;
 
-	if ((jwks_uri->signed_uri != NULL) && (jwks_uri->jwk != NULL)) {
+	if ((jwks_uri->signed_uri != NULL) && (jwks_uri->jwk_list != NULL)) {
 
 		oidc_jwt_t *jwt = NULL;
 		oidc_jose_error_t err;
 		apr_hash_t *keys = apr_hash_make(r->pool);
-		apr_hash_set(keys, jwks_uri->jwk->kid ? jwks_uri->jwk->kid : "", APR_HASH_KEY_STRING, jwks_uri->jwk);
+
+		oidc_debug(r, "signed_jwks verifier keys count=%d", jwks_uri->jwk_list->nelts);
+		for (int i = 0; i < jwks_uri->jwk_list->nelts; i++) {
+			oidc_jwk_t *jwk = APR_ARRAY_IDX(jwks_uri->jwk_list, i, oidc_jwk_t *);
+			if (jwk->kid != NULL) {
+				oidc_debug(r, "signed_jwks verifier kid=%s", jwk->kid);
+				apr_hash_set(keys, jwk->kid, APR_HASH_KEY_STRING, jwk);
+			} else {
+				const char *kid = apr_psprintf(r->pool, "%d", apr_hash_count(keys));
+				oidc_debug(r, "signed_jwks verifier kid=%s", kid);
+				apr_hash_set(keys, kid, APR_HASH_KEY_STRING, jwk);
+			}
+		}
 
 		if (oidc_jwt_parse(r->pool, response, &jwt, keys, FALSE, &err) == FALSE) {
 			oidc_error(r, "parsing JWT failed: %s", oidc_jose_e2s(r->pool, err));
@@ -597,7 +609,7 @@ static apr_byte_t oidc_metadata_jwks_retrieve_and_cache(request_rec *r, oidc_cfg
 		}
 
 		// TODO: add issuer?
-		if (oidc_proto_validate_jwt(r, jwt, NULL, TRUE, FALSE, -1) == FALSE)
+		if (oidc_proto_validate_jwt(r, jwt, NULL, FALSE, FALSE, -1) == FALSE)
 			return FALSE;
 
 		oidc_debug(r, "successfully verified and validated JWKs JWT");
@@ -668,9 +680,9 @@ apr_byte_t oidc_metadata_provider_retrieve(request_rec *r, oidc_cfg *cfg, const
 	OIDC_METRICS_TIMING_START(r, cfg);
 
 	/* get provider metadata from the specified URL with the specified parameters */
-	if (oidc_util_http_get(r, url, NULL, NULL, NULL, cfg->provider.ssl_validate_server, response,
-			       &cfg->http_timeout_short, &cfg->outgoing_proxy, oidc_dir_cfg_pass_cookies(r), NULL, NULL,
-			       NULL) == FALSE) {
+	if (oidc_http_get(r, url, NULL, NULL, NULL, cfg->provider.ssl_validate_server, response, NULL,
+			  &cfg->http_timeout_short, &cfg->outgoing_proxy, oidc_dir_cfg_pass_cookies(r), NULL, NULL,
+			  NULL) == FALSE) {
 		OIDC_METRICS_COUNTER_INC(r, cfg, OM_PROVIDER_METADATA_ERROR);
 		return FALSE;
 	}
@@ -740,10 +752,11 @@ apr_byte_t oidc_metadata_provider_get(request_rec *r, oidc_cfg *cfg, const char
 	}
 
 	/* assemble the URL to the .well-known OpenID metadata */
-	const char *url = apr_psprintf(r->pool, "%s",
-				       ((strstr(issuer, "http://") == issuer) || (strstr(issuer, "https://") == issuer))
-					   ? issuer
-					   : apr_psprintf(r->pool, "https://%s", issuer));
+	const char *url =
+	    apr_psprintf(r->pool, "%s",
+			 ((_oidc_strstr(issuer, "http://") == issuer) || (_oidc_strstr(issuer, "https://") == issuer))
+			     ? issuer
+			     : apr_psprintf(r->pool, "https://%s", issuer));
 	url =
 	    apr_psprintf(r->pool, "%s%s.well-known/openid-configuration", url,
 			 (url && url[_oidc_strlen(url) - 1] != OIDC_CHAR_FORWARD_SLASH) ? OIDC_STR_FORWARD_SLASH : "");
@@ -1077,9 +1090,12 @@ void oidc_metadata_get_valid_int(request_rec *r, const json_t *json, const char
 	oidc_json_object_get_int(json, key, &v, default_int_value);
 	const char *rv = valid_int_function(r->pool, v);
 	if (rv != NULL) {
-		oidc_warn(r, "integer value %d for key \"%s\" is invalid: %s; using default: %d", v, key, rv,
-			  default_int_value);
-		v = default_int_value;
+		// assume the provided default is valid, i.e. "unset" -1, so nothing to warn about
+		if (v != default_int_value) {
+			oidc_warn(r, "integer value %d for key \"%s\" is invalid: %s; using default: %d", v, key, rv,
+				  default_int_value);
+			v = default_int_value;
+		}
 	}
 	*int_value = v;
 }
@@ -1091,13 +1107,13 @@ static void oidc_metadata_get_jwks(request_rec *r, json_t *json, apr_array_heade
 	oidc_jwk_t *jwk = NULL;
 	json_t *elem = NULL;
 
-	keys = json_object_get(json, OIDC_JWK_KEYS);
+	keys = json_object_get(json, OIDC_JOSE_JWKS_KEYS_STR);
 	if (keys == NULL)
 		return;
 
 	if (!json_is_array(keys)) {
 		oidc_error(r, "trying to parse a list of JWKs but the value for key \"%s\" is not a JSON array",
-			   OIDC_JWK_KEYS);
+			   OIDC_JOSE_JWKS_KEYS_STR);
 		return;
 	}
 
@@ -1116,6 +1132,28 @@ static void oidc_metadata_get_jwks(request_rec *r, json_t *json, apr_array_heade
 	}
 }
 
+static void oidc_metadata_get_signed_jwks_uri_key(request_rec *r, json_t *j_conf, apr_array_header_t **jwk_list,
+						  apr_array_header_t *default_jwk_list) {
+	oidc_jose_error_t err;
+	json_t *json = json_object_get(j_conf, "signed_jwks_uri_key");
+	if (oidc_is_jwk(json)) {
+		oidc_jwk_t *jwk = NULL;
+		if (oidc_jwk_parse_json(r->pool, json, &jwk, &err) != TRUE) {
+			oidc_warn(r, "oidc_metadata_get_signed_jwks_uri_key failed: %s", oidc_jose_e2s(r->pool, err));
+			return;
+		}
+		*jwk_list = apr_array_make(r->pool, 1, sizeof(oidc_jwk_t *));
+		APR_ARRAY_PUSH(*jwk_list, oidc_jwk_t *) = jwk;
+	} else if (oidc_is_jwks(json)) {
+		if (oidc_jwks_parse_json(r->pool, json, jwk_list, &err) != TRUE) {
+			oidc_warn(r, "oidc_metadata_get_signed_jwks_uri_key failed: %s", oidc_jose_e2s(r->pool, err));
+			return;
+		}
+	} else if (default_jwk_list != NULL) {
+		*jwk_list = default_jwk_list;
+	}
+}
+
 /*
  * parse the JSON conf metadata in to a oidc_provider_t struct
  */
@@ -1126,16 +1164,7 @@ apr_byte_t oidc_metadata_conf_parse(request_rec *r, oidc_cfg *cfg, json_t *j_con
 
 	oidc_metadata_get_jwks(r, j_conf, &provider->client_keys);
 
-	oidc_jose_error_t err;
-	json_t *jwk = json_object_get(j_conf, "signed_jwks_uri_key");
-	if (jwk != NULL) {
-		if (oidc_jwk_parse_json(r->pool, jwk, &provider->jwks_uri.jwk, &err) == FALSE) {
-			oidc_warn(r, "oidc_jwk_parse_json failed for \"signed_jwks_uri_key\": %s",
-				  oidc_jose_e2s(r->pool, err));
-		}
-	} else if (cfg->provider.jwks_uri.jwk != NULL) {
-		provider->jwks_uri.jwk = cfg->provider.jwks_uri.jwk;
-	}
+	oidc_metadata_get_signed_jwks_uri_key(r, j_conf, &provider->jwks_uri.jwk_list, cfg->provider.jwks_uri.jwk_list);
 
 	/* get the (optional) signing & encryption settings for the id_token */
 	oidc_metadata_get_valid_string(r, j_conf, OIDC_METADATA_ID_TOKEN_SIGNED_RESPONSE_ALG,
@@ -1201,10 +1230,10 @@ apr_byte_t oidc_metadata_conf_parse(request_rec *r, oidc_cfg *cfg, json_t *j_con
 
 	/* get the PKCE method to use */
 	char *pkce_method = NULL;
+	/* NB: pass "none" rather than NULL to allow fallback when the default in base is explicitly set to "none" */
 	oidc_metadata_get_valid_string(r, j_conf, OIDC_METADATA_PKCE_METHOD, oidc_valid_pkce_method, &pkce_method,
-				       cfg->provider.pkce ? cfg->provider.pkce->method : NULL);
-	if (pkce_method != NULL)
-		oidc_parse_pkce_type(r->pool, pkce_method, &provider->pkce);
+				       cfg->provider.pkce ? cfg->provider.pkce->method : "none");
+	oidc_parse_pkce_type(r->pool, pkce_method, &provider->pkce);
 
 	/* get the client name */
 	oidc_json_object_get_string(r->pool, j_conf, OIDC_METADATA_CLIENT_NAME, &provider->client_name,
