@@ -321,9 +321,7 @@ static inline char *oidc_metrics_storage_get(server_rec *s) {
  * retrieve environment variable integer with default setting
  */
 static inline int oidc_metrics_get_env_int(const char *name, int dval) {
-	int v;
-	const char *env = getenv(name);
-	return (((env) && (sscanf(env, "%d", &v) == 1)) ? v : dval);
+	return _oidc_str_to_int(getenv(name), dval);
 }
 
 #define OIDC_METRICS_CACHE_JSON_MAX_ENV_VAR "OIDC_METRICS_CACHE_JSON_MAX"
@@ -660,8 +658,8 @@ static void *APR_THREAD_FUNC oidc_metrics_thread_run(apr_thread_t *thread, void
 		oidc_metrics_store(s);
 
 		/* reset the local hashtables */
-		apr_hash_clear(_oidc_metrics.counters);
-		apr_hash_clear(_oidc_metrics.timings);
+		oidc_util_apr_hash_clear(_oidc_metrics.counters);
+		oidc_util_apr_hash_clear(_oidc_metrics.timings);
 
 		/* unlock the mutex that protects the locally cached metrics */
 		oidc_cache_mutex_unlock(s->process->pool, s, _oidc_metrics_process_mutex);
@@ -1048,7 +1046,7 @@ static int oidc_metrics_handle_json(request_rec *r, char *s_json) {
 end:
 
 	/* return the data to the caller */
-	return oidc_util_http_send(r, s_json, _oidc_strlen(s_json), OIDC_CONTENT_TYPE_JSON, OK);
+	return oidc_http_send(r, s_json, _oidc_strlen(s_json), OIDC_HTTP_CONTENT_TYPE_JSON, OK);
 }
 
 /*
@@ -1057,7 +1055,7 @@ end:
 static int oidc_metrics_handle_internal(request_rec *r, char *s_json) {
 	if (s_json == NULL)
 		return HTTP_NOT_FOUND;
-	return oidc_util_http_send(r, s_json, _oidc_strlen(s_json), OIDC_CONTENT_TYPE_JSON, OK);
+	return oidc_http_send(r, s_json, _oidc_strlen(s_json), OIDC_HTTP_CONTENT_TYPE_JSON, OK);
 }
 
 #define OIDC_METRICS_SERVER_PARAM "server_name"
@@ -1075,9 +1073,9 @@ static int oidc_metrics_handle_status(request_rec *r, char *s_json) {
 	const char *s_key = NULL, *s_name = NULL;
 	void *iter = NULL;
 
-	oidc_util_get_request_parameter(r, OIDC_METRICS_SERVER_PARAM, &s_server);
-	oidc_util_get_request_parameter(r, OIDC_METRICS_COUNTER_PARAM, &metric);
-	oidc_util_get_request_parameter(r, OIDC_METRICS_SPEC_PARAM, &spec);
+	oidc_http_request_parameter_get(r, OIDC_METRICS_SERVER_PARAM, &s_server);
+	oidc_http_request_parameter_get(r, OIDC_METRICS_COUNTER_PARAM, &metric);
+	oidc_http_request_parameter_get(r, OIDC_METRICS_SPEC_PARAM, &spec);
 
 	if (s_server == NULL)
 		s_server = "localhost";
@@ -1115,7 +1113,7 @@ end:
 	if (json)
 		json_decref(json);
 
-	return oidc_util_http_send(r, msg, _oidc_strlen(msg), "text/plain", OK);
+	return oidc_http_send(r, msg, _oidc_strlen(msg), "text/plain", OK);
 }
 
 /*
@@ -1282,8 +1280,7 @@ static int oidc_metrics_handle_prometheus(request_rec *r, char *s_json) {
 
 	json_decref(json);
 
-	return oidc_util_http_send(r, ctx.s_result, _oidc_strlen(ctx.s_result), OIDC_METRICS_PROMETHEUS_CONTENT_TYPE,
-				   OK);
+	return oidc_http_send(r, ctx.s_result, _oidc_strlen(ctx.s_result), OIDC_METRICS_PROMETHEUS_CONTENT_TYPE, OK);
 }
 
 /*
@@ -1318,7 +1315,7 @@ static int oidc_metric_reset(request_rec *r, int dvalue) {
 	char svalue[16];
 	int value = 0;
 
-	oidc_util_get_request_parameter(r, OIDC_METRICS_RESET_PARAM, &s_reset);
+	oidc_http_request_parameter_get(r, OIDC_METRICS_RESET_PARAM, &s_reset);
 
 	if (s_reset == NULL)
 		return dvalue;
@@ -1343,7 +1340,7 @@ const oidc_metrics_content_handler_t *oidc_metrics_find_handler(request_rec *r)
 	int i = 0;
 
 	/* get the specified format */
-	oidc_util_get_request_parameter(r, OIDC_METRICS_FORMAT_PARAM, &s_format);
+	oidc_http_request_parameter_get(r, OIDC_METRICS_FORMAT_PARAM, &s_format);
 
 	if (s_format == NULL)
 		return &_oidc_metrics_handlers[0];
