@@ -155,7 +155,7 @@ static apr_byte_t oidc_session_load_cache(request_rec *r, oidc_session_t *z) {
 	apr_byte_t rc = FALSE;
 
 	/* get the cookie that should be our uuid/key */
-	char *uuid = oidc_util_get_cookie(r, oidc_cfg_dir_cookie(r));
+	char *uuid = oidc_http_get_cookie(r, oidc_cfg_dir_cookie(r));
 
 	/* get the string-encoded session from the cache based on the key; decryption is based on the cache backend
 	 * config */
@@ -172,7 +172,7 @@ static apr_byte_t oidc_session_load_cache(request_rec *r, oidc_session_t *z) {
 		/* cache backend does not contain an entry for the given key */
 		if (z->state == NULL) {
 			/* delete the session cookie */
-			oidc_util_set_cookie(r, oidc_cfg_dir_cookie(r), "", 0, OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r));
+			oidc_http_set_cookie(r, oidc_cfg_dir_cookie(r), "", 0, OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r));
 		}
 	}
 
@@ -198,15 +198,13 @@ static apr_byte_t oidc_session_save_cache(request_rec *r, oidc_session_t *z, apr
 		char *s_value = NULL;
 		if (oidc_session_encode(r, c, z, &s_value, FALSE) == FALSE)
 			return FALSE;
-		rc = oidc_cache_set_session(r, z->uuid, s_value, z->expiry);
 
+		rc = oidc_cache_set_session(r, z->uuid, s_value, z->expiry);
 		if (rc == TRUE)
 			/* set the uuid in the cookie */
-			oidc_util_set_cookie(
+			oidc_http_set_cookie(
 			    r, oidc_cfg_dir_cookie(r), z->uuid, c->persistent_session_cookie ? z->expiry : -1,
-			    c->cookie_same_site
-				? (first_time ? OIDC_COOKIE_EXT_SAME_SITE_LAX : OIDC_COOKIE_EXT_SAME_SITE_STRICT)
-				: OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r));
+			    c->cookie_same_site ? OIDC_COOKIE_EXT_SAME_SITE_LAX : OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r));
 
 	} else {
 
@@ -214,7 +212,7 @@ static apr_byte_t oidc_session_save_cache(request_rec *r, oidc_session_t *z, apr
 			oidc_cache_set_sid(r, z->sid, NULL, 0);
 
 		/* clear the cookie */
-		oidc_util_set_cookie(r, oidc_cfg_dir_cookie(r), "", 0, OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r));
+		oidc_http_set_cookie(r, oidc_cfg_dir_cookie(r), "", 0, OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r));
 
 		/* remove the session from the cache */
 		rc = oidc_cache_set_session(r, z->uuid, NULL, 0);
@@ -227,7 +225,7 @@ static apr_byte_t oidc_session_save_cache(request_rec *r, oidc_session_t *z, apr
  * load the session from a self-contained client-side cookie
  */
 static apr_byte_t oidc_session_load_cookie(request_rec *r, oidc_cfg *c, oidc_session_t *z) {
-	char *cookieValue = oidc_util_get_chunked_cookie(r, oidc_cfg_dir_cookie(r), c->session_cookie_chunk_size);
+	char *cookieValue = oidc_http_get_chunked_cookie(r, oidc_cfg_dir_cookie(r), c->session_cookie_chunk_size);
 	if ((cookieValue != NULL) && (oidc_session_decode(r, c, z, cookieValue, TRUE) == FALSE))
 		return FALSE;
 	return TRUE;
@@ -242,27 +240,35 @@ static apr_byte_t oidc_session_save_cookie(request_rec *r, oidc_session_t *z, ap
 	if ((z->state != NULL) && (oidc_session_encode(r, c, z, &cookieValue, TRUE) == FALSE))
 		return FALSE;
 
-	oidc_util_set_chunked_cookie(
-	    r, oidc_cfg_dir_cookie(r), cookieValue, c->persistent_session_cookie ? z->expiry : -1,
-	    c->session_cookie_chunk_size,
-	    (z->state == NULL)	  ? OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r)
-	    : c->cookie_same_site ? (first_time ? OIDC_COOKIE_EXT_SAME_SITE_LAX : OIDC_COOKIE_EXT_SAME_SITE_STRICT)
-				  : OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r));
+	oidc_http_set_chunked_cookie(r, oidc_cfg_dir_cookie(r), cookieValue,
+				     c->persistent_session_cookie ? z->expiry : -1, c->session_cookie_chunk_size,
+				     (z->state == NULL)	   ? OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r)
+				     : c->cookie_same_site ? OIDC_COOKIE_EXT_SAME_SITE_LAX
+							   : OIDC_COOKIE_EXT_SAME_SITE_NONE(c, r));
 
 	return TRUE;
 }
 
+static inline int oidc_session_get_int(request_rec *r, oidc_session_t *z, const char *key, int def_val) {
+	int v;
+	oidc_json_object_get_int(z->state, key, &v, def_val);
+	return v;
+}
+
+static inline apr_time_t oidc_session_get_key2timestamp(request_rec *r, oidc_session_t *z, const char *key) {
+	int value = -1;
+	oidc_json_object_get_int(z->state, key, &value, -1);
+	return (value > -1) ? apr_time_from_sec(value) : -1;
+}
+
 apr_byte_t oidc_session_extract(request_rec *r, oidc_session_t *z) {
 	apr_byte_t rc = FALSE;
 
 	if (z->state == NULL)
 		goto out;
 
-	json_t *j_expires = json_object_get(z->state, OIDC_SESSION_EXPIRY_KEY);
-	if (j_expires)
-		z->expiry = apr_time_from_sec(json_integer_value(j_expires));
-
 	/* check whether it has expired */
+	z->expiry = oidc_session_get_key2timestamp(r, z, OIDC_SESSION_EXPIRY_KEY);
 	if (apr_time_now() > z->expiry) {
 
 		oidc_warn(r, "session restored from cache has expired");
@@ -314,6 +320,17 @@ apr_byte_t oidc_session_load(request_rec *r, oidc_session_t **zz) {
 	return rc;
 }
 
+static void oidc_session_set_int(request_rec *r, oidc_session_t *z, const char *key, int v) {
+	if (z->state == NULL)
+		z->state = json_object();
+	json_object_set_new(z->state, key, json_integer(v));
+}
+
+static void oidc_session_set_timestamp(request_rec *r, oidc_session_t *z, const char *key, const apr_time_t timestamp) {
+	if (timestamp > -1)
+		oidc_session_set_int(r, z, key, apr_time_sec(timestamp));
+}
+
 /*
  * save a session to cache/cookie
  */
@@ -324,7 +341,7 @@ apr_byte_t oidc_session_save(request_rec *r, oidc_session_t *z, apr_byte_t first
 
 	if (z->state != NULL) {
 		oidc_session_set(r, z, OIDC_SESSION_REMOTE_USER_KEY, z->remote_user);
-		json_object_set_new(z->state, OIDC_SESSION_EXPIRY_KEY, json_integer(apr_time_sec(z->expiry)));
+		oidc_session_set_timestamp(r, z, OIDC_SESSION_EXPIRY_KEY, z->expiry);
 		oidc_session_set(r, z, OIDC_SESSION_SESSION_ID, z->uuid);
 	}
 
@@ -431,11 +448,6 @@ apr_byte_t oidc_session_set(request_rec *r, oidc_session_t *z, const char *key,
  */
 typedef const char *(*oidc_session_get_str_function)(request_rec *r, oidc_session_t *z);
 
-static void oidc_session_set_timestamp(request_rec *r, oidc_session_t *z, const char *key, const apr_time_t timestamp) {
-	if (timestamp != -1)
-		oidc_session_set(r, z, key, apr_psprintf(r->pool, "%" APR_TIME_T_FMT, apr_time_sec(timestamp)));
-}
-
 static json_t *oidc_session_get_str2json(request_rec *r, oidc_session_t *z,
 					 oidc_session_get_str_function session_get_str_fn) {
 	json_t *json = NULL;
@@ -451,14 +463,6 @@ static const char *oidc_session_get_key2string(request_rec *r, oidc_session_t *z
 	return s_value;
 }
 
-static apr_time_t oidc_session_get_key2timestamp(request_rec *r, oidc_session_t *z, const char *key) {
-	apr_time_t t_expires = 0;
-	const char *s_expires = oidc_session_get_key2string(r, z, key);
-	if (s_expires != NULL)
-		sscanf(s_expires, "%" APR_TIME_T_FMT, &t_expires);
-	return apr_time_from_sec(t_expires);
-}
-
 #define OIDC_SESSION_WARN_CLAIM_SIZE 1024 * 8
 #define OIDC_SESSION_WARN_CLAIM_SIZE_VAR "OIDC_SESSION_WARN_CLAIM_SIZE"
 
@@ -480,7 +484,7 @@ void oidc_session_set_filtered_claims(request_rec *r, oidc_session_t *z, const c
 	if (r->subprocess_env != NULL) {
 		s = apr_table_get(r->subprocess_env, OIDC_SESSION_WARN_CLAIM_SIZE_VAR);
 		if (s) {
-			warn_claim_size = _oidc_str_to_int(s);
+			warn_claim_size = _oidc_str_to_int(s, OIDC_SESSION_WARN_CLAIM_SIZE);
 			oidc_debug(r, "warn_claim_size set to %d in environment variable %s", warn_claim_size,
 				   OIDC_SESSION_WARN_CLAIM_SIZE_VAR);
 		}
@@ -593,14 +597,20 @@ const char *oidc_session_get_access_token(request_rec *r, oidc_session_t *z) {
  * access token expires
  */
 void oidc_session_set_access_token_expires(request_rec *r, oidc_session_t *z, const int expires_in) {
-	if (expires_in != -1) {
-		oidc_session_set(r, z, OIDC_SESSION_KEY_ACCESSTOKEN_EXPIRES,
-				 apr_psprintf(r->pool, "%" APR_TIME_T_FMT, apr_time_sec(apr_time_now()) + expires_in));
+	if (expires_in > -1) {
+		oidc_debug(r, "storing access token expires_in in the session: %d", expires_in);
+		oidc_session_set_timestamp(r, z, OIDC_SESSION_KEY_ACCESSTOKEN_EXPIRES,
+					   apr_time_now() + apr_time_from_sec(expires_in));
 	}
 }
 
-const char *oidc_session_get_access_token_expires(request_rec *r, oidc_session_t *z) {
-	return oidc_session_get_key2string(r, z, OIDC_SESSION_KEY_ACCESSTOKEN_EXPIRES);
+apr_time_t oidc_session_get_access_token_expires(request_rec *r, oidc_session_t *z) {
+	return oidc_session_get_key2timestamp(r, z, OIDC_SESSION_KEY_ACCESSTOKEN_EXPIRES);
+}
+
+const char *oidc_session_get_access_token_expires2str(request_rec *r, oidc_session_t *z) {
+	apr_time_t expires = oidc_session_get_access_token_expires(r, z);
+	return (expires > -1) ? apr_psprintf(r->pool, "%" APR_TIME_T_FMT, apr_time_sec(expires)) : NULL;
 }
 
 /*
@@ -641,11 +651,11 @@ const char *oidc_session_get_cookie_domain(request_rec *r, oidc_session_t *z) {
  */
 
 void oidc_session_set_userinfo_refresh_interval(request_rec *r, oidc_session_t *z, const int interval) {
-	oidc_session_set_timestamp(r, z, OIDC_SESSION_KEY_USERINFO_REFRESH_INTERVAL, apr_time_from_sec(interval));
+	oidc_session_set_int(r, z, OIDC_SESSION_KEY_USERINFO_REFRESH_INTERVAL, interval);
 }
 
-apr_time_t oidc_session_get_userinfo_refresh_interval(request_rec *r, oidc_session_t *z) {
-	return oidc_session_get_key2timestamp(r, z, OIDC_SESSION_KEY_USERINFO_REFRESH_INTERVAL);
+int oidc_session_get_userinfo_refresh_interval(request_rec *r, oidc_session_t *z) {
+	return oidc_session_get_int(r, z, OIDC_SESSION_KEY_USERINFO_REFRESH_INTERVAL, -1);
 }
 
 void oidc_session_reset_userinfo_last_refresh(request_rec *r, oidc_session_t *z) {
@@ -659,8 +669,8 @@ apr_time_t oidc_session_get_userinfo_last_refresh(request_rec *r, oidc_session_t
 /*
  * access_token last refresh
  */
-void oidc_session_reset_access_token_last_refresh(request_rec *r, oidc_session_t *z) {
-	oidc_session_set_timestamp(r, z, OIDC_SESSION_KEY_ACCESS_TOKEN_LAST_REFRESH, apr_time_now());
+void oidc_session_set_access_token_last_refresh(request_rec *r, oidc_session_t *z, apr_time_t ts) {
+	oidc_session_set_timestamp(r, z, OIDC_SESSION_KEY_ACCESS_TOKEN_LAST_REFRESH, ts);
 }
 
 apr_time_t oidc_session_get_access_token_last_refresh(request_rec *r, oidc_session_t *z) {
